/*
 */
#ifndef _GPIB_GUT_HXX_
#define _GPIB_GUT_HXX_

#include <iostream>
#include "cplRaw/failable.hxx"
#include "umi/umi_user.hxx"
#include "gpib/gumi_user.hxx"
#include "gpibc/gumi_msgs.hxx"

/**
 * \file
 * GPIB interface test classes
 */

namespace INdepGpib100 {

  namespace Umi=INdepUmi100;


  /**
   * GPIB user base class (Test)
   *
   * This class allows to access the GPIB controller hardware
   * registers directly. Do not use except for manufacturing tests.
   */
  class GpibTest:public Failable {
    Umi::Umi *_pum;


    ErrE _eErr;				/* result of last call	*/

    Umi::Umi *pum() const { return _pum; }
    const Gpib_gti_Proto _pt;		/* protocol instance		*/


   public:
    //friend std::ostream &operator <<( std::ostream &os, const GpibTest &lr );
    bool operator!() const { return pum()->isFail(); }


    int peek(int i);			/**< read hardware register byte  */
    Failable &poke(int i, int x);	/**< write hardware register byte */

    /** check bits in loop */
    int checkBits( int iReg, int fBits, int fMask, int fOr, int cRetry, int *piLbr, int *piFbr  );

    Failable &ping(GpibInfo*);
    

    /** constructor, uses \ref puf to create UMI channel */
    GpibTest(Umi::Factory *puf);	

    /**
     * Constructor to be used with a specific Umi object.
     */
    GpibTest(Umi::Umi *pum);	
    ~GpibTest() ;
  };

  /*
   * construction wrapper
   */
  class GpibInterfaceTest:public Failable {
    GpibTest *_pGut;
     

   public:
    GpibTest *pGut() { return _pGut; }


    GpibInterfaceTest(const char *sz);
    ~GpibInterfaceTest();
  };
}

#endif
