/************************************************************

  Declarations for the various GPIB controllers

************************************************************/

#ifndef _IGPIBREG_H
#define _IGPIBREG_H


/* register bank 0; NEC compatible registers */
#define IGC_BANK0             0	/* start of register bank 0 */
#define IGC_DIO               0	/* (r/w) data in/ byte out */
#define IGC_I1	              1	/* (r/w) interrupt status/mask 1 */
#define IGC_I2	              2	/* (r/w) interrupt status/mask 2 */
#define IGC_SP	              3	/* (r/w) serial poll status/mode */
#define IGC_ADSTS             4	/* (r) address status */
#define IGC_ADMOD             4	/* (w) address mode */
#define IGC_CPT	              5	/* (r) Command pass through */
#define IGC_AUXMOD            5	/* (w) auxiliary mode */
#define IGC_AD0	              6	/* (r) address 0 */
#define IGC_AD01              6	/* (w) address 0/1 */
#define IGC_AD1	              7	/* (r) address 1 */
#define IGC_EOS1              7	/* (w) EOS1 byte */

/* register bank 1 */
#define IGC_BANK1             8	/* start of register bank 1 */
#define IGC_FFSTS             8	/* (r) FIFO status */
#define IGC_XDMA              8	/* (w) extended DMA control */
#define IGC_I3	              9	/* (r/w) interrupt status/mask 3 */
#define IGC_I4	             10	/* (r/w) interrupt status/mask 4 */
#define IGC_TCNTLO           11	/* (r/w) terminal count low byte */
#define IGC_TCNTHI           12	/* (r/w) terminal count high byte */
#define IGC_REVC             13	/* (r) revision code */
#define IGC_TMOLO            13	/* (w) timeout low byte */
#define IGC_OEMC             14	/* (r) oem code */
#define IGC_TMOHI            14	/* (w) timeout high byte */
#define IGC_XSTS             15	/* (r) extended status */
#define IGC_XMOD             15	/* (w) extended mode */

/* register bank 2 */
#define IGC_BANK2            16	/* start of register bank 2 */
#define IGC_IFFCNT           16	/* (r) input FIFO count */
#define IGC_IFFWM            16	/* (w) input FIFO watermark */
#define IGC_OFFCNT           17	/* (r) output FIFO count */
#define IGC_OFFWM            17	/* (w) output FIFO watermark */
#define IGC_BMDTA            18	/* (r/w) bus monitor data */
#define IGC_BMCTL            19	/* (r/w) bus monitor ctrl */
#define IGC_BMDIR            20	/* (w) bus monitor direction */
#define IGC_SRQEN            21	/* (w) srq enable */
#define IGC_TSTRG            22	/* (r) test register */
#define IGC_EOS2             22	/* (w) EOS 2 byte */
#define IGC_EOS3             23	/* (w) EOS 3 byte */

/* interrupt register 1 */
#define IGC_I1_DI	   	   0x01	/* data in */
#define IGC_I1_DO		   0x02	/* data out */
#define IGC_I1_ERR	   0x04	/* error: byte out cont. lost */
#define IGC_I1_DEC	   0x08	/* device clear */
#define IGC_I1_END	   0x10	/* end message */
#define IGC_I1_DET	   0x20	/* device trigger */
#define IGC_I1_APT	   0x40	/* address pass through */
#define IGC_I1_CPT	   0x80	/* command pass through */

/* interrupt register 2 */
#define IGC_I2_ADSC	   0x01	/* address changed */
#define IGC_I2_REMC	   0x02	/* remote changed */
#define IGC_I2_LOKC	   0x04	/* lok changed */
#define IGC_I2_CO   	   0x08	/* command out */
#define IGC_I2_REM 	   0x10	/* remote (r) */
#define IGC_I2_DMAI	   0x10	/* DMA on input (w) */
#define IGC_I2_LOK 	   0x20	/* lokal (r) */
#define IGC_I2_DMAO	   0x20	/* DMA on output (w) */
#define IGC_I2_SRQI	   0x40	/* service request */
#define IGC_I2_INT	   0x80	/* any of these interrupts */

/* extended interrupt register 3 */
#define IGC_I3_TMO	   0x01	/* timer expiration */
#define IGC_I3_TC	     	   0x02	/* terminal count */
#define IGC_I3_CMD	   0x04	/* command received */
#define IGC_I3_TCT	   0x08	/* take control received */
#define IGC_I3_IFC	   0x10	/* IFC has been set */
#define IGC_I3_ATN	   0x20	/* atn has been set */
#define IGC_I3_FFERR	   0x40	/* error on FIFO */

/* extended interrupt register 4 */
#define IGC_I4_IFWM	   0x01	/* input FIFO reached watermark */
#define IGC_I4_OFWM	   0x02	/* output FIFO reached watermark */
#define IGC_I4_IFFL	   0x04	/* input FIFO full */
#define IGC_I4_OFEM	   0x08	/* output FIFO empty */
#define IGC_I4_IFRDY	   0x10	/* input FIFO ready */
#define IGC_I4_OFRDY	   0x20	/* output FIFO ready */
#define IGC_I4_NIFWM	   0x40	/* input FIFO under watermark */
#define IGC_I4_NOFWM	   0x80	/* output FIFO under watermark */

/* igpib auxiliary commands (IGC_AUX) */
#define IGC_AUXCMD_IPO	   0x00 /* immideate exectute pon */
#define IGC_AUXCMD_RES	   0x02	/* chip reset */
#define IGC_AUXCMD_FH	   0x03	/* finish handshake */
#define IGC_AUXCMD_TRG	   0x04	/* trigger (pulse trig pin) */
#define IGC_AUXCMD_RTL	   0x05	/* return to local (pulse mode ) */
#define IGC_AUXCMD_SRTL	   0x0d	/* return to local (set 'rtl') */
#define IGC_AUXCMD_FEOI	   0x06	/* send EOI */
#define IGC_AUXCMD_ANV	   0x07	/* address not valid */
#define IGC_AUXCMD_AV	   0x0f	/* address valid */
#define IGC_AUXCMD_PPFC	   0x01	/* clear ppoll flag: use srq */
#define IGC_AUXCMD_PPFS	   0x09	/* set ppoll flag: use ppoll */
#define IGC_AUXCMD_GTS	   0x10	/* go to standby */
#define IGC_AUXCMD_TCA	   0x11	/* take control async */
#define IGC_AUXCMD_TCS	   0x12	/* take control sync */
#define IGC_AUXCMD_TCE	   0x1a	/* take control sync on end */
#define IGC_AUXCMD_LTN	   0x13	/* listen, gen 'ltn' pulse */
#define IGC_AUXCMD_LTC	   0x1b	/* listen in continous mode */
#define IGC_AUXCMD_LUN	   0x1c	/* local unlisten */
#define IGC_AUXCMD_EPP	   0x1d	/* execute parallel poll */
#define IGC_AUXCMD_CIFC	   0x16	/* clear ifc */
#define IGC_AUXCMD_CREN	   0x17	/* clear remote enable */
#define IGC_AUXCMD_GOTMS   0x18 /* goto TMS mode */
#define IGC_AUXCMD_SIFC    0x1e /* set ifc */
#define IGC_AUXCMD_SREN	   0x1f	/* set remote enable */
#define IGC_AUXCMD_DSC	   0x14	/* disable system control */
#define IGC_AUXCMD_XMOD	   0x0a	/* switch to extended mode */

/* auxuliary register offsets */
#define IGC_AUXOFF_CLK	   0x20	/*  refclock */
#define IGC_AUXOFF_AUXD	   0x40	/*  write to aux D */
#define IGC_AUXOFF_WPP	   0x60	/*  write ppol reg */
#define IGC_AUXOFF_AUXA	   0x80	/*  write aux A */
#define IGC_AUXOFF_AUXB	   0xa0	/*  write aux B */
#define IGC_AUXOFF_AUXE	   0xc0	/*  write aux E */
#define IGC_AUXOFF_AUXF	   0xe0	/*  write ext. aux reg */

/* auxiliary register A */
#define IGC_AUXA_NHS	   0x00	/* normal handshake mode */
#define IGC_AUXA_HAM	   0x01	/* rfd holdoff on all data */
#define IGC_AUXA_HEM	   0x02	/* rfd holdoff on end mode */
#define IGC_AUXA_CM	   0x03	/* continous mode */
#define IGC_AUXA_LEOS	   0x04	/* enable end at eos (listen) */
#define IGC_AUXA_TEOS	   0x08	/* enable end at eos (talk) */
#define IGC_AUXA_EOSB	   0x10	/* enable 8-Bit eos detection */

/* auxiliary register B */
#define IGC_AUXB_UDCPT	   0x01	/* set CPT during detection of undef cmds */
#define IGC_AUXB_SPEND	   0x02	/* permit the transm. of end during spoll */
#define IGC_AUXB_HISP	   0x04	/* highspeed on write data */
#define IGC_AUXB_INTLVL	   0x08	/* set active level of int pin */
#define IGC_AUXB_ISTSRQ	   0x10	/* set ist to SRQ ppoll otherwise */

/* aux register E */
#define IGC_AUXE_CLR	   0x01	/* holdoff on clear */
#define IGC_AUXE_TRG	   0x02	/* holdoff on trigger */
#define IGC_AUXE_ALL	   0x04	/* holdoff on all cmds */
#define IGC_AUXE_TCT	   0x08	/* holdoff on take control */

/* aux register F */
#define IGC_AUXF_DISL      0x00 /* disable listener function */
#define IGC_AUXF_ENAL	   0x01	/* enable listener function */
#define IGC_AUXF_MAP1	   0x02	/* map registerset 1 on 0 */
#define IGC_AUXF_MAP2	   0x03	/* map registerset 2 on 0 */
#define IGC_AUXF_RFDH	   0x04	/* set RFD Holdoff */
#define IGC_AUXF_OFFCLR	   0x05	/* output FIFO counter clear */
#define IGC_AUXF_IFFCLR	   0x06	/* input FIFO counter clear */

/* address status register (IGC_ADSTS) */
#define IGC_ADSTS_MJMN	   0x01	/* major/minor address */
#define IGC_ADSTS_TA	   0x02	/* talker activer state */
#define IGC_ADSTS_LA	   0x04	/* listener active */
#define IGC_ADSTS_TPAS	   0x08	/* talker primary addressed state */
#define IGC_ADSTS_LPAS	   0x10	/* listener primary addressed state */
#define IGC_ADSTS_SPMS	   0x20	/* serial poll mode state */
#define IGC_ADSTS_NATN	   0x40	/* attention */
#define IGC_ADSTS_CIC	   0x80	/* controller in charge */

/* address mode register (IGC_ADMOD) */
#define IGC_ADMOD_ADM0	   0x01	/* addr mode minor bit */
#define IGC_ADMOD_ADM1	   0x02	/* addr mode major bit */
#define IGC_ADMOD_OFENA	   0x04	/* output FIFO enable (256 byte) */
#define IGC_ADMOD_IFENA	   0x08	/* input FIFO enable */
#define IGC_ADMOD_TRM0	   0x10	/* transmit/receive mode 0 */
#define IGC_ADMOD_TRM1	   0x20	/* transmit/receive mode 1 */
#define IGC_ADMOD_LON	   0x40	/* listen only */
#define IGC_ADMOD_TON	   0x80	/* talk only */

/* address register 0 (IGC_AD0) */
#define IGC_AD0_DLAD	   0x20	/* (r/w) disable listener address */
#define IGC_AD0_DTAD	   0x40	/* (r/w) disable talker address */
#define IGC_AD0_ADSEL	   0x80	/* (w) address register select */
#define IGC_AD0_T8RUN	   0x80	/* (r) set by ifc/ren and cleared by timer */

/* address register 1 (IGC_AD1) */
#define IGC_AD1_DLAD	   0x20	/* (r) disable listener address */
#define IGC_AD1_DTAD	   0x40	/* (r) disable talker address */
#define IGC_AD1_EOI	   0x80	/* (r) latched eoi of current byte in FIFO */

/* extended DMA control (IGC_XDMA) */
#define IGC_XDMA_OUT       0x01	/* extended DMA output (GPIB talk) */
#define IGC_XDMA_ENA	   0x02	/* extended DMA enable */
#define IGC_XDMA_SPD	   0x04	/* DMA speed set-> IO on every clock tick
				   else on every 2. clock tick */
#define IGC_XDMA_AUXO	   0x08	/* DMA width; set 16-Bit */

/* extended status register (IGC_XSTS) */
#define IGC_XSTS_OQMU	   0x01	/* output queue message unit in prog */
#define IGC_XSTS_SCSEL	   0x02	/* SCSEL pin on 68 pin chip */
#define IGC_XSTS_LIDIS	   0x04	/* listener mode disabled */
#define IGC_XSTS_IFEM	   0x08	/* input FIFO is empty */
#define IGC_XSTS_OFFL	   0x10	/* output FIFO is full */

/* extended mode register (IGC_XMOD) */
#define IGC_XMOD_TRGEN	   0x01	/* enable trigger on T/R3 Pin */
#define IGC_XMOD_MAVEN	   0x02	/* reset MAV on end */
#define IGC_XMOD_E1ENA	   0x04	/* enable extended EOS1 */
#define IGC_XMOD_E2ENA	   0x08	/* enable extended EOS2 */
#define IGC_XMOD_EOIDIS	   0x10	/* ignore eoi */
#define IGC_XMOD_TCENA	   0x20	/* enable terminal count */
#define IGC_XMOD_TCTLK	   0x40	/* use terminal count on talk */
#define IGC_XMOD_TCLBH	   0x80	/* enable last byte handling */

/* bus monitor control lines */
#define IGC_BMCTL_DAV	   0x01	/* data valid */
#define IGC_BMCTL_NRFD	   0x02	/* ready for data */
#define IGC_BMCTL_NDAC	   0x04	/* data accepted */
#define IGC_BMCTL_IFC	   0x08	/* ifc line */
#define IGC_BMCTL_ATN	   0x10	/* atn line */
#define IGC_BMCTL_SRQ	   0x20	/* srq line */
#define IGC_BMCTL_REN	   0x40	/* remote line */
#define IGC_BMCTL_EOI	   0x80	/* eoi line */

/* test register */
#define IGC_TSTRG_SRQS	   0x01	/* service request state */

/* bus monitor write control */
#define IGC_BMDIR_DTA	   0x01	/* set to data as output */
#define IGC_BMDIR_SC	   0x02	/* system controller */
#define IGC_BMDIR_EOIOE	   0x04	/* set EOI as output */
#define IGC_BMDIR_CIC	   0x08
#define IGC_BMDIR_PE	   0x10
#define IGC_BMDIR_ENA	   0x80	/* enable for output */


/* TMS kompatible registers, bits and commands */

#define TMS_I0                0	/* (r/w) interrupt status/mask 1*/
#define TMS_I1	              1	/* (r/w) interrupt status/mask 2 */
#define TMS_ADSTS	      2	/* (r) address status */
#define TMS_BUS	              3	/* (r) bus status */
#define TMS_AUXMOD            3	/* (w) auxiliary mode */
#define TMS_AD	              4	/* (w) address mode */
#define TMS_SP                5	/* (w) seriall poll */
#define TMS_CPT	              6	/* (r) Command pass through */
#define TMS_PP                6	/* (w) parallel poll */
#define TMS_DIO	              7	/* (r/w) data in/out */

/* interrupt register 1 */
#define TMS_I0_MAC	   0x01	/* my address change */
#define TMS_I0_RLC	   0x02	/* remote local change */
#define TMS_I0_SPAS	   0x04	/* serial poll sent */
#define TMS_I0_END	   0x08	/* end message */
#define TMS_I0_BO	   0x10 /* data out */
#define TMS_I0_BI	   0x20	/* data in */
#define TMS_I0_INT1  	   0x40	/* interrupt in INT1 */
#define TMS_I0_INT0	   0x80	/* interrupt in INT0 */

/* interrupt register 2 */
#define TMS_I1_IFC	   0x01	/* interface clear received */
#define TMS_I1_SRQ	   0x02	/* service request received */
#define TMS_I1_MA	   0x04	/* My Address rec.*/
#define TMS_I1_DCAS	   0x08	/* device clear active state */
#define TMS_I1_APT 	   0x10	/* Address pass through */
#define TMS_I1_UNC 	   0x20	/* unrecognized command */
#define TMS_I1_ERR	   0x40	/* No listener found */
#define TMS_I1_GET	   0x80	/* Trigger received */

/* address status register (TMS_ADSTS) */
#define TMS_ADSTS_ULPA	   0x01	/* LSB of last recog. address */
#define TMS_ADSTS_TADS	   0x02	/* talker addressed/active state */
#define TMS_ADSTS_LADS	   0x04	/* listener addr./active state */
#define TMS_ADSTS_TPAS	   0x08	/* talker primary addressed state */
#define TMS_ADSTS_LPAS	   0x10	/* listener primary addressed state */
#define TMS_ADSTS_ATN	   0x20	/* attention is true */
#define TMS_ADSTS_LLO	   0x40	/* lockout in operation */
#define TMS_ADSTS_REM	   0x80	/* device is in remote state */

/* address mode register */
#define TMS_AD_DTAD	   0x20	/* (r/w) disable talker address */
#define TMS_AD_DLAD	   0x40	/* (r/w) disable listener address */
#define TMS_AD_EDPA	   0x80	/* enable dual primary addr mode */

/* auxiliary command register */
#define TMS_AUXCMD_TRUE	   0x80	/* set option to true */

#define TMS_AUXCMD_SWRES   0x00 /* Software reset */
#define TMS_AUXCMD_DACR	   0x01	/* release DAC holdoff */
#define TMS_AUXCMD_RHDF	   0x02	/* release RFD holdoff */
#define TMS_AUXCMD_HDFA	   0x03	/* holdoff on all data */
#define TMS_AUXCMD_HDFE	   0x04	/* holdoff on EOI only */
#define TMS_AUXCMD_NBAF	   0x05	/* new byte avaliable false */
#define TMS_AUXCMD_FGET	   0x06	/* force group execute trigger */
#define TMS_AUXCMD_RTL	   0x07	/* return to local */
#define TMS_AUXCMD_FEOI	   0x08	/* send EOI with next byte */
#define TMS_AUXCMD_LON	   0x09	/* listen only */
#define TMS_AUXCMD_TON	   0x0a	/* talk only */
#define TMS_AUXCMD_GTS	   0x0b	/* go to standby */
#define TMS_AUXCMD_TCA	   0x0c	/* Take control asynchronously */
#define TMS_AUXCMD_TCS	   0x0d	/* take control synchronously */
#define TMS_AUXCMD_RPP	   0x0e	/* request parallel poll */
#define TMS_AUXCMD_SIC	   0x0f	/* send interface clear */
#define TMS_AUXCMD_SRE	   0x10	/* send remote enable */
#define TMS_AUXCMD_RQC	   0x11	/* request control */
#define TMS_AUXCMD_RLC	   0x12	/* release control */
#define TMS_AUXCMD_DAI	   0x13	/* disable all interrupts */
#define TMS_AUXCMD_PTS	   0x14	/* pass through next secondary */
#define TMS_AUXCMD_STDL	   0x15	/* short t1 settling time */
#define TMS_AUXCMD_SHDW	   0x16	/* shadow handshake */
#define TMS_AUXCMD_VSTDL   0x17	/* very short T1 delay */
#define TMS_AUXCMD_RSV2	   0x18	/* request service bit 2 */

#define TMS_BUS_REN	   0x01	/* remote line */
#define TMS_BUS_IFC	   0x02	/* ifc line */
#define TMS_BUS_SRQ	   0x04	/* srq line */
#define TMS_BUS_EOI	   0x08	/* eoi line */
#define TMS_BUS_NRFD	   0x10	/* ready for data */
#define TMS_BUS_NDAC	   0x20	/* data accepted */
#define TMS_BUS_DAV	   0x40	/* data valid */
#define TMS_BUS_ATN	   0x80	/* atn line */

/* GP5130 >= 1.5 */

/*
 * 5130 register set
 */
#define GP5130_I1      	   IGC_I1
#define GP5130_I2      	   IGC_I2
#define GP5130_AD1     	   IGC_AD1
#define GP5130_DIO     	   IGC_DIO
#define GP5130_AUXMOD      IGC_AUXMOD
#define GP5130_RFFLO       0x08  // (r) read fifo count lo byte
#define GP5130_RTCLO       0x08  // (w) read transfer count lo byte
#define GP5130_RXSTS       0x09  // (r) extended read status
#define GP5130_RXMOD       0x09  // (w) extended read mode
#define GP5130_WFFLO       0x0a  // (r) write fifo count lo byte
#define GP5130_WTCLO       0x0a  // (w) write transfer count lo byte
#define GP5130_WXMOD       0x0b  // (w) extended write mode
#define GP5130_WXSTS       0x0b  // (r) extended write status
#define GP5130_GPCIN       0x0c  // (r) gpib control lines (inputs)
#define GP5130_GPCOUT	   0x0d  // (r) gpib control lines (outputs)
#define GP5130_VERS	   0x0e  // (r) version register
#define GP5130_XSTS	   0x0f  // (r) extended status
#define GP5130_XMOD	   0x0f  // (w) extended mode

#define G72100_CCFG	   0x0e  // (r) core config register

#define G72100_CCFG_CV	   0xe0		/* core version	  	 */
#define G72100_CCFG_CV_SR  5		/* shift right		 */
#define G72100_CCFG_CEN	   0x10		/* ctl enabled	  	 */
#define G72100_CCFG_F9	   0x04		/* 2^9 byte fifo  	 */
#define G72100_CCFG_XE	   0x01		/* extended mode enable  */
#define G72100_CCFG_X2     0x02		/* tbd			 */
#define G72100_CCFG_X4	   0x04		/* tbd			 */


/* 
 * 5130 auxiliary commands 
 */
#define GP5130_AUXCMD_FH   IGC_AUXCMD_FH	 
#define GP5130_AUXCMD_XMOD IGC_AUXCMD_XMOD


// auxiliary register offsets
//---------------------------
#define GP5130_AUXOFF_CLK	 0x20	//  refclock
#define GP5130_AUXOFF_AUXD 0x40	//  write to aux D
#define GP5130_AUXOFF_WPP	 0x60	//  write ppol reg
#define GP5130_AUXOFF_AUXA 0x80	//  write aux A
#define GP5130_AUXOFF_AUXB 0xa0	//  write aux B
#define GP5130_AUXOFF_AUXE 0xc0	//  write aux E
#define GP5130_AUXOFF_AUXF 0xe0	//  write ext. aux reg

// auxiliary register A
//---------------------
#define GP5130_AUXA_NHS	   0x00	// normal handshake mode
#define GP5130_AUXA_HAM	   0x01	// rfd holdoff on all data
#define GP5130_AUXA_HEM	   0x02	// rfd holdoff on end mode
#define GP5130_AUXA_CM	   0x03	// continous mode
#define GP5130_AUXA_LEOS	 0x04	// enable end at eos (listen)
#define GP5130_AUXA_TEOS	 0x08	// enable end at eos (talk)
#define GP5130_AUXA_EOSB	 0x10	// enable 8-Bit eos detection

// auxiliary register B
//---------------------
#define GP5130_AUXB_UDCPT	 0x01	// set CPT during detection of undef cmds
#define GP5130_AUXB_SPEND	 0x02	// permit the transm. of end during spoll
#define GP5130_AUXB_HISP	 0x04	// highspeed on write data
#define GP5130_AUXB_INTLVL 0x08	// set active level of int pin
#define	GP5130_AUXB_ISTSRQ 0x10	// set ist to SRQ ppoll otherwise

#if 0
// aux register E
#define GP5130_AUXE_CLR	   0x01	// holdoff on clear
#define GP5130_AUXE_TRG	   0x02	// holdoff on trigger
#define GP5130_AUXE_ALL	   0x04	// holdoff on all cmds
#define GP5130_AUXE_TCT	   0x08	// holdoff on take control

// aux register F
#define GP5130_AUXF_DISL	 0x00	// disable listener function
#define GP5130_AUXF_ENAL	 0x01	// enable listener function
#define GP5130_AUXF_MAP1	 0x02	// map registerset 1 on 0
#define GP5130_AUXF_MAP2	 0x03	// map registerset 2 on 0
#define GP5130_AUXF_RFDH	 0x04	// set RFD Holdoff
#define GP5130_AUXF_OFFCLR 0x05	// output FIFO counter clear
#define GP5130_AUXF_IFFCLR 0x06	// input FIFO counter clear
#endif //0


// address status register
//------------------------
#define GP5130_ADSTS_MJMN	 0x01	// major/minor address
#define GP5130_ADSTS_TA	   0x02	// talker activer state
#define GP5130_ADSTS_LA	   0x04	// listener active
#define GP5130_ADSTS_TPAS	 0x08	// talker primary addressed state
#define GP5130_ADSTS_LPAS	 0x10	// listener primary addressed state
#define GP5130_ADSTS_SPMS	 0x20	// serial poll mode state
#define GP5130_ADSTS_NATN	 0x40	// attention
#define GP5130_ADSTS_CIC	 0x80	// controller in charge

// address mode register
//----------------------
#define GP5130_ADMOD_ADM0	 0x01	// addr mode minor bit
#define GP5130_ADMOD_ADM1	 0x02	// addr mode major bit
//#define	GP5130_ADMOD_OFENA0x04	// output FIFO enable (256 byte)
//#define GP5130_ADMOD_IFENA0x08	// input FIFO enable
#define GP5130_ADMOD_TRM0	 0x10	// transmit/receive mode 0
#define GP5130_ADMOD_TRM1	 0x20	// transmit/receive mode 1
#define GP5130_ADMOD_LON	 0x40	// listen only
#define GP5130_ADMOD_TON	 0x80	// talk only

// address register 0
//-------------------
#define GP5130_AD0_DLAD	   0x20	// (r/w) disable listener address
#define GP5130_AD0_DTAD	   0x40	// (r/w) disable talker address
#define GP5130_AD0_ADSEL	 0x80	// (w) address register select
//#define GP5130_AD0_T8RUN 0x80	// (r) set by ifc/ren and cleared by timer

// address register 1
//-------------------
#define GP5130_AD1_DLAD	   0x20	// (r) disable listener address
#define GP5130_AD1_DTAD	   0x40	// (r) disable talker address
#define GP5130_AD1_EOI	   0x80	// (r) latched eoi of current byte in FIFO


// gpib control lines (GPCIN/GPCOUT)
//----------------------------------
#define GP5130_GPC_NRFD    0x01
#define GP5130_GPC_NDAC    0x02
#define GP5130_GPC_DAV     0x04
#define GP5130_GPC_EOI     0x08
#define GP5130_GPC_SRQ     0x10
#define GP5130_GPC_ATN     0x20
#define GP5130_GPC_REN     0x40
#define GP5130_GPC_IFC     0x80


// extended read/write mode/status
//--------------------------------
#define GP5130_XRW_ENTC    0x10 // (w)   enable transfer count
#define GP5130_XRW_FFCH    0x01 // (r)   fifo count bit 9
#define GP5130_XINT_TC     0x20 // (r/w) transfer count reached int bit/mask
#define GP5130_XINT_EMPT   0x40 // (r/w) fifo empty int bit/mask
#define GP5130_XINT_FULL   0x80 // (r/w) fifo full int bit/mask


// extended mode/status
//---------------------
#define GP5130_XMOD_HDF    0x01 // (w)   set RFD holdoff
#define GP5130_XSTS_RDY    0x01 // (r)   internal 'ready' msg
#define GP5130_XSTS_RTC    0x02 // (r)   read terminal count reached
#define GP5130_XSTS_REMPT  0x04 // (r)   read fifo empty
#define GP5130_XSTS_RFULL  0x08 // (r)   read fifo full
#define GP5130_XSTS_NBA    0x10 // (r)   internal 'new byte available' msg
#define GP5130_XSTS_WTC    0x20 // (r)   write terminal count reached
#define GP5130_XSTS_WEMPT  0x40 // (r)   write fifo empty
#define GP5130_XSTS_WFULL  0x80 // (r)   write fifo full

#endif
