#include <linux/types.h>
#include "fixme.h"
#include "umic/umiad.h"
#include "card.h"
#include "mod.h"
#include "assert.h"
#include <linux/sched.h>
#include <linux/usb.h>
#include <linux/version.h>
#include <linux/interrupt.h>

 /*
  * usb.c
  *
  * driver for USB adaptors
  */
  #define SZ_USBSTAT MAX_CACN

  /* usb channel transfer flags */

  #define US_DAV  0x1   /* data available	*/
  #define US_CAV  0x2   /* control available	*/
  #define US_DEM  0x4 	/* empty data part	*/
  #define US_CEM  0x8   /* empty control part	*/
  #define US_UPS  0xf   /* all upstream flags	*/

  #define US_CAC  0x10  /* will accept control  */
  #define US_DAC  0x20  /* will accept data	*/
  #define US_CEND 0x40  /* command done		*/
  #define US_DEND 0x80  /* data done		*/

  /* per channel status flags */

  typedef struct {
    Card _ca;				/* header			*/

    /*
     * Adaptor information
     */
    UmiAdaptUsb *_pad;
    int _fOpen;				/* open flag			*/

    /*
     * current endpoint status
     */
    u_char _mpchStat[SZ_USBSTAT];

    struct usb_device *_pud;
    size_t _mpSepPut[MAX_CACN];		
    size_t _mpSepGet[MAX_CACN];		
    int _mpIepPut[MAX_CACN];		/* per channel PUT endpoint	*/
    int _mpIepGet[MAX_CACN];		/* per channel GET endpoint	*/

  } CardUsb;

  #define setBusy(p) 		/* set/clr busy flag	*/
  #define clrBusy(p) 
  #define pcu_pca(p) ((&(p)->_ca))
  #define pcu_pdm(p) (pca_pdm(&(p)->_ca))
  #define pcu_sep(p, i, m) ((p)->_mpSep##m[(i)])
  #define pcu_iep(p, i, m) ((p)->_mpIep##m[(i)])


  /* 
   * firmware loader
   */
  struct ezusb_hex_record {
	  __u16 _addr;
	  __u8 _cData;
	  __u8 _data[16];
  };

  /*
   * ezusb_writememory ()
   */
  int ezusb_writememory (struct usb_device *pud, int addr, unsigned char *pchData, int length, __u8 bRequest) {
    int result;
    unsigned char *pch;

    /* dbg("ezusb_writememory %x, %d", addr, length); */

    pch =  kmalloc (length, GFP_KERNEL);
    if (!pch) {
      //printk(KERN_ERR PFX "kmalloc(%d) failed\n", pcu_pdm(pcu)->_szDid, length);
      return -ENOMEM;
    }
    memcpy (pch, pchData, length);
    result = usb_control_msg (pud, usb_sndctrlpipe(pud, 0), bRequest, 0x40, addr, 0, pch, length, 3*HZ);
    //printk(KERN_DEBUG "result(%d)\n", result);
    kfree (pch);
    return result;
  }



  /*
   * ezusb_set_reset()
   */
  int ezusb_set_reset (struct usb_device *pud, unsigned char reset_bit) {
    int	response;
    /* write CPUCS register */
    response = ezusb_writememory (pud, 0xe600, &reset_bit, 1, 0xa0);
    return response;
  }


  /*
   * putDown control part in kernel space
   */
  int usbPutDownControlCore(CardUsb *pcu, Chan *pcc, struct UmiControlStruct *puc, char *pchBuf,  size_t sBuf ) {
    size_t c;
    int l;
    int e;


    /* marshal the data */
    c=UmiProto_marshal(pcc->_pUdd->_pup, pchBuf, sBuf, puc);

    l=0;
    if (isDebug(pcu_pdm(pcu))) printk(KERN_DEBUG PFX "usb put down ctrl, chan=%d, size=%zu h=%d\n", pcu_pdm(pcu)->_szDid, pcc->_iChanOfDriver, c, *((unsigned char*)pchBuf));
    e = usb_bulk_msg (pcu->_pud, usb_sndbulkpipe(pcu->_pud, pcu_iep(pcu,pcc->_iChanOfDriver,Put)), pchBuf, c, &l,  HZ);
    if (isDebug(pcu_pdm(pcu))) printk(KERN_DEBUG PFX "usb bulk send %zu %d\n", pcu_pdm(pcu)->_szDid, c,e);
    return e;
  }

  /*
   * putDown control part in user space
   */
  int usbPutDownControl(CardUsb *pcu, Chan *pcc, struct UmiMemStruct *pud, char *pchBuf,  size_t sBuf ) {
    char mpch[SZ_CTRL_MAX];
    size_t c;
    int e;

    /* copy down the control message */
    if ((e=bufPutDown(pud, mpch, SZ_CTRL_MAX, &c))) return e;
    assert(c > 0);	/* control never zero length */
    return usbPutDownControlCore(pcu, pcc, (struct UmiControlStruct *)mpch, pchBuf,  sBuf );
  }

  /*
   * putDown data
   */
  int usbPutDownDataCore(CardUsb *pcu, Chan *pcc, char *pchBuf,  size_t c) {
    int l;
    int e;
    if (c==0) {
      /* REMIND: can't sent zero length */
      memset(pchBuf, 0, 2);
      c=2;
    }

    if (isDebug(pcu_pdm(pcu))) printk(KERN_DEBUG PFX "usb put down data size=%zu\n", pcu_pdm(pcu)->_szDid, c);
    e = usb_bulk_msg (pcu->_pud, usb_sndbulkpipe(pcu->_pud, pcu_iep(pcu,pcc->_iChanOfDriver,Put)), pchBuf, c, &l,  HZ);
    if (isDebug(pcu_pdm(pcu))) printk(KERN_DEBUG PFX "usb bulk send %zu %d\n", pcu_pdm(pcu)->_szDid, c,e);
    return e;
  }

  int usbPutDownData(CardUsb *pcu, Chan *pcc, struct UmiMemStruct *pud, char *pchBuf,  size_t sBuf) {
    size_t c;
    int e;
    if ((e=bufPutDown(pud, pchBuf, sBuf, &c))) return e;
    return usbPutDownDataCore(pcu, pcc, pchBuf, c);

  }

  /*
   * get up control
   */
  int usbGetUpControl(CardUsb *pcu, Chan*pcc, struct UmiMemStruct *pud, char *pchBuf,  size_t sBuf) {
    char mpch[SZ_CTRL_MAX];
    int e;

    size_t c;
    int l;

    e = usb_bulk_msg (pcu->_pud, usb_rcvbulkpipe(pcu->_pud, pcu_iep(pcu,pcc->_iChanOfDriver,Get)), pchBuf, sBuf, &l,  HZ*10);
    if (isDebug(pcu_pdm(pcu))) printk(KERN_DEBUG PFX "usb bulk recv %d %d\n", pcu_pdm(pcu)->_szDid, l,e);
    if (e) return e;

    if (isDebug(pcu_pdm(pcu))) {
      int i;
      printk(KERN_DEBUG PFX "data ", pcu_pdm(pcu)->_szDid);
      for (i=0; i<l; i++) {
	printk("%02x ", (((unsigned char *)pchBuf)[i]));
      }
      printk("\n");
    }

    /* unmarsh the packet to local buffer */
    c=UmiProto_unmarsh(pcc->_pUdd->_pup, (struct UmiControlStruct *)mpch, pchBuf, l);
    if (c == 0) {
      printk(KERN_ERR PFX "can't unmarsh h=%d l=%d\n", pcu_pdm(pcu)->_szDid, *((unsigned char*)pchBuf),l);
      return -EPROTO;
    }

    /* copy data part to the user */
    e=bufGetUp(pud, mpch, c);
    if (e) printk(KERN_ERR PFX "can't copy up %zu %d\n", pcu_pdm(pcu)->_szDid, c,e);
    if (isDebug(pcu_pdm(pcu))) printk(KERN_DEBUG PFX "usb getup ctrl size=%zu\n", pcu_pdm(pcu)->_szDid, c);
    return e;
  }

  int usbGetUpData(CardUsb *pcu, Chan*pcc, struct UmiMemStruct *pud, char *pchBuf,  size_t sBuf) {
    int e;
    int l;

    e = usb_bulk_msg (pcu->_pud, usb_rcvbulkpipe(pcu->_pud, pcu_iep(pcu,pcc->_iChanOfDriver,Get)), pchBuf, sBuf, &l,  HZ*100);
    if (isDebug(pcu_pdm(pcu))) printk(KERN_DEBUG PFX "usb bulk recv %d %d\n", pcu_pdm(pcu)->_szDid, l,e);
    if (e) return e;

    /* copy the part to the user */
    e=bufGetUp(pud, pchBuf, l);
    if (e) printk(KERN_ERR PFX "can't copy up %d %d\n", pcu_pdm(pcu)->_szDid, l,e);
    if (isDebug(pcu_pdm(pcu))) printk(KERN_DEBUG PFX "usb getup data size=%d\n", pcu_pdm(pcu)->_szDid, l);
    return e;
  }

  /*
   * put down
   */
  int usbPutDown(Card *pca, Chan *pcc, unsigned long arg) {
    int iChan=pcc->_iChanOfCard;
    int e=0;
    char *pch;
    UmiIoc *pubU=(void*)arg;
    CardUsb *pcu=(CardUsb*)pca;
    assert(iChan >= 0 && iChan < MAX_CACN);

    /*
     * get a USB-able transfer buffer
     */
    pch =  kmalloc (max(SZ_CTRL_MAX, SZ_DATA_MAX), GFP_KERNEL);
    if (!pch) return -ENOMEM;

    if ((e=usbPutDownControl(pcu, pcc, &pubU->_cd, pch, SZ_CTRL_MAX))) goto lClean;
    if ((e=usbPutDownData(pcu, pcc, &pubU->_dd, pch, SZ_DATA_MAX))) goto lClean;

   lClean:
    kfree(pch);
    return e;
  }

  /*
   * put down UMI layer  message
   */
  int usbPutDownUmiLayer(Card *pca, Chan *pcc, unsigned eType) {
    int iChan=pcc->_iChanOfCard;
    int e=0;
    char *pch;
    CardUsb *pcu=(CardUsb*)pca;
    assert(iChan >= 0 && iChan < MAX_CACN);

    /*
     * get a USB-able transfer buffer
     */
    pch =  kmalloc (max(SZ_CTRL_MAX, SZ_DATA_MAX), GFP_KERNEL);
    if (!pch) return -ENOMEM;
    struct UmiControlStruct ucs;
    ucs._eType=eType;
    ucs._size=0;

    if ((e=usbPutDownControlCore(pcu, pcc, &ucs, pch, SZ_CTRL_MAX))) goto lClean;
    if ((e=usbPutDownDataCore(pcu, pcc, pch, 0))) goto lClean;

   lClean:
    kfree(pch);
    return e;
  }


  int usbGetUp(Card *pca, Chan *pcc, unsigned long arg) {
    int iChan=pcc->_iChanOfCard;
    CardUsb *pcu=(CardUsb*)pca;
    register int e=0;
    char *pch=0;
    UmiIoc *pubU=(void*)arg;
    assert(iChan >= 0 && iChan < MAX_CACN);

    /*
     * get a USB-able transfer buffer
     */
    pch= kmalloc(max(SZ_CTRL_MAX, SZ_DATA_MAX), GFP_KERNEL);
    if (!pch) return -ENOMEM;
    /* get data first */
    if ((e=usbGetUpData(pcu, pcc, &pubU->_du, pch, SZ_DATA_MAX))) goto lClean;
    if ((e=usbGetUpControl(pcu, pcc, &pubU->_cu, pch, SZ_CTRL_MAX))) goto lClean;

   lClean:
    kfree(pch);
    return e;

  }

  int usbDownload(CardUsb *pcu, const struct ezusb_hex_record *phr) {
    int e;
    assert(phr);

    /* firmware download */
    if ((e=ezusb_set_reset (pcu->_pud, 1))<0) {
      printk(KERN_ERR PFX "usb can't reset (%d)\n", pcu_pdm(pcu)->_szDid, e);
      return -1;
    }
    while(phr->_addr != 0xffff) {
      e = ezusb_writememory(pcu->_pud, phr->_addr, (unsigned char *)phr->_data, phr->_cData, 0xa0);
      if (e < 0) {
	printk(KERN_ERR PFX "usb can't load firmware (%d %04X %p %d)\n", pcu_pdm(pcu)->_szDid, e, phr->_addr, phr->_data, phr->_cData);
	return -1;
      }
      phr++;
    }
    if ((e=ezusb_set_reset (pcu->_pud, 0))<0) {
      printk(KERN_ERR PFX "usb can't restart (%d)\n", pcu_pdm(pcu)->_szDid, e);
      return -1;
    }
    printk(KERN_INFO PFX "usb download done\n", pcu_pdm(pcu)->_szDid);
    return 0;
  }

  int usbOpen(Card *pca, Chan *pcc) {
    int iChanOfCard=pcc->_iChanOfCard;
    CardUsb *pcu=(CardUsb*)pca;

    if (pcu->_fOpen == 0) {
      const struct ezusb_hex_record *phr=pcu->_pad->_mppfw[pcc->_iDriverOfCard];
      /* first open of any channel */
      if (!phr) return -ENODEV;

      if (usbDownload(pcu, phr)) return -ENODEV;

      /* delay for init the ezusb to start and initialize */
      udelay(1000);

    }

    pcu->_fOpen |= (1 << iChanOfCard);
    return 0;
  }

  int usbClose(Card *pca, Chan *pcc) {
    int iChanOfCard=pcc->_iChanOfCard;
    CardUsb *pcu=(CardUsb*)pca;
    assert(iChanOfCard >= 0 && iChanOfCard < MAX_CACN);

    pcu->_fOpen &= ~(1 << iChanOfCard);

    if (pcu->_fOpen == 0) {
      /*
       * load idle firmware
       */
      usbDownload(pcu, pcu->_pad->_pfwIdle);
    }
    return 0;
  }

  char* usbPrint( Card *pca, char* pchBuf, int c) {
    return pchBuf;
  }


  /*
   * Card construction/destruction
   */
  CardUsb *usbCardDelete(LumiMod *pdm, CardUsb *pcu) {
    if (pcu) kfree (pcu);
    return 0;
  }

  CardUsb *usbCardNew(LumiMod *pdm) {
    CardUsb *pcu=(CardUsb*)kmalloc(sizeof(CardUsb), GFP_KERNEL);
    if (!pcu) return 0;

    pca_init(&pcu->_ca, pdm );
    pcu->_ca._pfPutDown=usbPutDown;
    pcu->_ca._pfGetUp=usbGetUp;
    pcu->_ca._pfOpen=usbOpen;
    pcu->_ca._pfClose=usbClose;
    pcu->_ca._pfPrint=usbPrint;
    pcu->_fOpen=0;
    memset(pcu->_mpchStat, 0, sizeof(pcu->_mpchStat));

    return pcu;
  }



  static char * szIdentUsb( char mzBuf[64], struct usb_device *pud ) {
    sprintf(mzBuf, "usb:");
    usb_make_path(pud, mzBuf+4, 60);
    return mzBuf;
  }




  int pdm_usb_probe(LumiMod *pdm, struct usb_interface *pui, const struct usb_device_id *pdi) {
    int i;
    int iSlot;
    CardUsb *pcu;
    struct usb_device *pud=interface_to_usbdev(pui);
    char mzPath[64];
    const char *szId=szIdentUsb(mzPath, pud);
    struct usb_host_interface *pUhi;
    struct usb_endpoint_descriptor *pUed;

    printk(KERN_INFO PFX "usb probe for %x:%x at %s\n", pdm->_szDid, pdi->idVendor, pdi->idProduct, szId);
    printk(KERN_INFO PFX "usb num_altsetting %d\n", pdm->_szDid, pui->num_altsetting);

    /* select altsetting to scan */
    if (pui->num_altsetting < 4) {
      printk(KERN_ERR PFX "invalid interface\n", pdm->_szDid);
      return -1;
    }
    pUhi = pui->cur_altsetting+1;

    if (pUhi->desc.bNumEndpoints <5) {
      printk(KERN_ERR PFX "wrong config\n", pdm->_szDid);
      return -1;
    }

    for (i = 0; i < pUhi->desc.bNumEndpoints; i++) {
      pUed = &pUhi->endpoint[i].desc;
      printk(KERN_INFO PFX " ep=%d attr=%x addr=%x bs=%d\n", pdm->_szDid, i, pUed->bmAttributes, pUed->bEndpointAddress, pUed->wMaxPacketSize);
    }

    if (usb_set_interface(pud, 0, 1)) {
      printk(KERN_ERR PFX "cant set interface/alt\n", pdm->_szDid);
      return -1;
    }


    /* get s slot */
    if ((iSlot=pdm_getslot(pdm, szId))==-1) return -1;

    /*
     * allocate and initialize a card structure
     */
    pcu=usbCardNew(pdm);
    if (!pcu) return -1;

    pcu->_pud=pud;
    pcu->_pad=(UmiAdaptUsb*)pdi->driver_info;

    /* 
     * configure pipes 
     * we have 2 bidir pipes, assigned to chan 0 and 1 of the driver
     */
    pcu->_mpSepPut[0] = pUhi->endpoint[2].desc.wMaxPacketSize;
    pcu->_mpSepPut[1] = pUhi->endpoint[3].desc.wMaxPacketSize;
    pcu->_mpSepGet[0] = pUhi->endpoint[4].desc.wMaxPacketSize;
    pcu->_mpSepGet[1] = pUhi->endpoint[5].desc.wMaxPacketSize;
    pcu->_mpIepPut[0] = pUhi->endpoint[2].desc.bEndpointAddress;
    pcu->_mpIepPut[1] = pUhi->endpoint[3].desc.bEndpointAddress;
    pcu->_mpIepGet[0] = pUhi->endpoint[4].desc.bEndpointAddress;
    pcu->_mpIepGet[1] = pUhi->endpoint[5].desc.bEndpointAddress;


    /*
     * register the card
     */
    pdm_setcard(pdm, iSlot, &pcu->_ca);

    usbDownload(pcu, pcu->_pad->_pfwIdle);

    return 0;
  }

  void pdm_usb_disconnect(LumiMod*pdm, struct usb_interface *pui) {
    struct usb_device *pud=interface_to_usbdev(pui);
    char mpch[SZ_CARD_ID];

    int iSlot;
    if ((iSlot=pdm_findslot(pdm, szIdentUsb(mpch, pud)))!=-1) {
      CardUsb*pcu;
      if ((pcu=(CardUsb*)pdm_clrcard(pdm, iSlot))) {
	usbDownload(pcu, pcu->_pad->_pfwDiscon);
	printk(KERN_INFO PFX "usb release apaptor %s@%d\n", pdm->_szDid, mpch, iSlot);
	usbCardDelete(pdm, pcu);
      }
    }
  }
